import sys

if sys.platform.startswith("linux"):
    from opencanary.modules import CanaryService, FileSystemWatcher
    from opencanary.config import ConfigException

    import os, re, random, shutil, time
    from datetime import datetime

    class SambaLogWatcher(FileSystemWatcher):

        def __init__(self, logFile=None, logger=None):
            self.logger = logger
            FileSystemWatcher.__init__(self, fileName=logFile)

        def handleLines(self, lines=None):
            audit_re = re.compile(r'^.*smbd_audit:.*$')

            for line in lines:
                    matches = audit_re.match(line)

                    #Skip lines that do not match the correct RegEx pattern
                    if matches is None:
                           continue

                    data = line.split('smbd_audit:',1)[-1].strip().split('|')

                    user = data[0]
                    srcHost = data[1]
                    dstHost = data[2]
                    srcHostName = data[3]
                    shareName = data[4]
                    dstHostName = data[5]
                    smbVersion = data[6]
                    smbArch = data[7]
                    domainName = data[9]
                    auditAction = data[10]
                    auditStatus = data[11]
                    path = data[12]

                    if user == "":
                      user = "anonymous"

                    data = {}
                    data['src_host'] = srcHost
                    data['src_port'] = '-1'
                    data['dst_host'] = dstHost
                    data['dst_port'] = 445
                    data['logtype'] =  self.logger.LOG_SMB_FILE_OPEN
                    data['logdata'] = {'USER':user, 'REMOTENAME': srcHostName, 'SHARENAME': shareName,
                                       'LOCALNAME': dstHostName, 'SMBVER': smbVersion, 'SMBARCH': smbArch,
                                       'DOMAIN': domainName, 'AUDITACTION': auditAction,
                                       'STATUS':auditStatus, 'FILENAME': path}
                    self.logger.log(data)

    class CanarySamba(CanaryService):
        NAME = 'smb'
        def __init__(self,config=None, logger=None):
            CanaryService.__init__(self, config=config, logger=logger)
            self.audit_file = config.getVal('smb.auditfile', default='/var/log/samba-audit.log')
            self.config = config

        def startYourEngines(self, reactor=None):
            #create samba run dir, so testparm doesn't error
            #try:
            #    os.stat('/var/run/samba')
            #except OSError:
            #    os.mkdir('/var/run/samba')

            fs = SambaLogWatcher(logFile=self.audit_file, logger=self.logger)
            fs.start()



from opencanary.modules import CanaryService

from twisted.application import internet
from twisted.internet.protocol import DatagramProtocol

from twisted.internet.address import IPv4Address

from scapy.all import SNMP
"""
    A log-only SNMP server. It won't respond, but it will log SNMP queries.
"""

class MiniSNMP(DatagramProtocol):
    def datagramReceived(self, data, host_and_port):
        try:
            snmp = SNMP(data)
            community = snmp.community.val
            requests = [x.oid.val for x in snmp.PDU.varbindlist]

            logdata={'REQUESTS': requests, 'COMMUNITY_STRING': community}
            self.transport.getPeer = lambda: IPv4Address('UDP', host_and_port[0], host_and_port[1])
            self.factory.log(logdata=logdata, transport=self.transport)
        except Exception as e:
            print(e)
            pass


class CanarySNMP(CanaryService):
    NAME = 'SNMP'

    def __init__(self, config=None, logger=None):
        CanaryService.__init__(self, config=config, logger=logger)
        self.port = int(config.getVal('snmp.port', default=161))
        self.logtype = logger.LOG_SNMP_CMD
        self.listen_addr = config.getVal('device.listen_addr', default='')

    def getService(self):
        f = MiniSNMP()
        f.factory = self
        return internet.UDPServer(self.port, f, interface=self.listen_addr)



from opencanary.modules.ftp import CanaryFTP
from opencanary.config import config
from opencanary.logger import getLogger
from twisted.trial import unittest
from twisted.test import proto_helpers
from twisted.internet import reactor, defer

logger = getLogger(config)

class CanaryFTPTestCase(unittest.TestCase):

    def setUp(self):
        canaryFTP = CanaryFTP(config=config, logger=logger)
        # self.service = canaryFTP.getService()
        # self.service.startService()
        self.proto = canaryFTP.getService().args[1].buildProtocol(('127.0.0.1', config.getVal('ftp.port')))
        self.tr = proto_helpers.StringTransport()
        self.proto.makeConnection(self.tr)

    def _test(self, operation, a, b, expected):
        self.proto.dataReceived(u'{} {} {}\r\n'.format(operation, a, b).encode('utf-8'))
        self.assertEqual(int(self.tr.value()), expected)

    def test_banner(self):
        # import pdb;pdb.set_trace()
        return self.assertEqual(self.tr.value(), '220 {}\r\n'.format(config.getVal('ftp.banner')))

    # @defer.inlineCallbacks
    def tearDown(self):
        # result = yield self.service.stopService()
        self.proto.wrappedProtocol.setTimeout(None)

    # def test_logging(self):
    #     import pdb; pdb.set_trace()
    #     return True


    # def test_add(self):
    #     return self._test('add', 7, 6, 13)


    # def test_subtract(self):
    #     return self._test('subtract', 82, 78, 4)


    # def test_multiply(self):
    #     return self._test('multiply', 2, 8, 16)


    # def test_divide(self):
    #     return self._test('divide', 14, 3, 4)


import codecs
import os.path
from setuptools import setup, find_namespace_packages
import sys


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    """
    Reading the package version dynamically.
    https://packaging.python.org/en/latest/guides/single-sourcing-package-version/
    """
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

requirements = [
    'Twisted==22.8.0',
    'pyasn1==0.4.5',
    'cryptography==38.0.1',
    'simplejson==3.16.0',
    'requests==2.21.0',
    'zope.interface==5.0.0',
    'PyPDF2==1.26.0',
    'fpdf==1.7.2',
    'passlib==1.7.1',
    'Jinja2==3.0.1',
    'ntlmlib==0.72',
    'bcrypt==3.1.7',
    'setuptools==63.2.0',
    'hpfeeds==3.0.0',
    'pyOpenSSL==22.1.0',
    "service-identity==21.1.0"
]


setup(
    name='opencanary',
    version=get_version("opencanary/__init__.py"),
    url='http://www.thinkst.com/',
    author='Thinkst Applied Research',
    author_email='info@thinkst.com',
    description='OpenCanary daemon',
    long_description='A low interaction honeypot intended to be run on internal networks.',
    install_requires=requirements,
    license='BSD',
    packages=find_namespace_packages(
        exclude=[
            'docs','docs*'
            'opencanary.test','opencanary.test*'
        ]
    ),
    include_package_data = True,
    scripts=['bin/opencanaryd','bin/opencanary.tac'],
    platforms='any',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Framework :: Twisted",
        "Topic :: System :: Networking",
        "Topic :: Security",
        "Topic :: System :: Networking :: Monitoring",
        "Natural Language :: English",
        "Operating System :: Unix",
        "Operating System :: POSIX :: Linux",
        "Operating System :: POSIX :: BSD :: FreeBSD",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: BSD License",
    ],
)
